@@ -1,8 +1,8 @@
 ;;; honest-report.el --- make bug report with screenshot and keylog
 
-;; Copyright (c) 2005
+;; Copyright (c) 2005, 2006
 ;;   by HIRAOKA Kazuyuki <khi@users.sourceforge.jp>
-;; $Id: honest-report.el,v 1.3 2005/09/06 14:16:05 hira Exp $
+;; $Id: honest-report.el,v 1.5 2006/03/22 13:29:25 hira Exp $
 ;;
 ;; This program is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -83,9 +83,17 @@
                    (format "--- %s ---\n%s"
                            w
                            (buffer-substring-no-properties b e)))))
-             (window-list)
+             (honest-report-window-list)
              "\n"))
 
+(defun honest-report-window-list ()
+  "Mimic `window-list'.
+This function exists only for emacs20 (and meadow-1.15),
+which lack `window-list'."
+  (let ((ws nil))
+    (walk-windows (lambda (w) (setq ws (cons w ws))))
+    (reverse ws)))
+
 (defun honest-report-message ()
   (with-current-buffer (or (get-buffer "*Messages*")
                            (get-buffer " *Message-Log*"))
