@@ -1,7 +1,7 @@
 ;;; howm-backend.el --- Wiki-like note-taking tool
-;;; Copyright (c) 2005
+;;; Copyright (c) 2005, 2006
 ;;;   by HIRAOKA Kazuyuki <khi@users.sourceforge.jp>
-;;; $Id: howm-backend.el,v 1.20 2005/09/27 16:03:09 hira Exp $
+;;; $Id: howm-backend.el,v 1.23 2006/01/18 14:59:06 hira Exp $
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -28,6 +28,9 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; class Folder
 
+(defun howm-folder ()
+  (howm-make-folder:files (howm-search-path)))
+
 ;; * class Folder: abstraction of directory
 
 ;; (Wrong comments. Ignore me.)
@@ -176,16 +179,7 @@ DUMMY-EXCLUSION-CHECKER has no effect; it should be removed soon."
   (remove nil (mapcar #'howm-page-name (howm-folder-pages:pages folder))))
 
 (defun howm-folder-grep-internal:pages (folder pattern &optional fixed-p)
-  (let* ((pages (howm-folder-pages:pages folder))
-         (h nil))
-    ;; classify page types
-    (mapc (lambda (p)
-            (let* ((type (howm-page-type p))
-                   (c (assoc type h)))
-              (if (null c)
-                  (setq h (cons (cons type (list p)) h))
-                (setf (cdr c) (cons p (cdr c))))))
-          pages)
+  (let ((h (howm-classify #'howm-page-type (howm-folder-pages:pages folder) t)))
     ;; get result for each type
     (apply #'append (mapcar (lambda (p)
                               (let ((type (car p))
@@ -752,12 +746,14 @@ STR can be list of strings. They are regarded as 'or' pattern of all elements."
       (insert-file-contents page))))
 
 (defun howm-page-viewer:file (page)
-;;   (let ((dir-viewer (and (howm-folder-p page)
-  (let ((dir-viewer (and (file-directory-p page)
-                         (list #'find-file)))
-;;                          (list #'howm-folder-viewer)))
-        (viewer (cdr (howm-cl-assoc-if (lambda (reg) (string-match reg page))
-                                       howm-view-external-viewer-assoc))))
+  (let* ((ls (lambda (dir)
+               (with-temp-buffer
+                 (insert-directory dir "-l")
+                 (buffer-substring-no-properties (point-min) (point-max)))))
+         (dir-viewer (and (file-directory-p page)
+                          (howm-make-viewer:func #'find-file ls)))
+         (viewer (cdr (howm-cl-assoc-if (lambda (reg) (string-match reg page))
+                                        howm-view-external-viewer-assoc))))
     (or dir-viewer viewer
         (and howm-view-use-mailcap
              (let* ((ext (if (string-match "\\.[^\\.]+$" page)
@@ -866,6 +862,9 @@ STR can be list of strings. They are regarded as 'or' pattern of all elements."
 ;; Viewer is one of the following.
 ;; func    ==> (func) is called after (find-file page).
 ;; (func)  ==> (func page) is called.
+;; (func . previewer)
+;;   ==> (func page) and (previewer page) are called for open and preview
+;;   (previewer must return a string).
 ;; "str"   ==> (format "str" page) is externally executed on shell.
 
 (defun howm-viewer-type (viewer &rest r)
@@ -880,8 +879,8 @@ STR can be list of strings. They are regarded as 'or' pattern of all elements."
   (gfunc-def howm-viewer-indicator (viewer page))
 )
 
-(defun howm-make-viewer:func (f)
-  (list f))
+(defun howm-make-viewer:func (f &optional previewer)
+  (cons f previewer))
 
 (when howm-view-use-mailcap
   (require 'mailcap)
@@ -899,12 +898,20 @@ STR can be list of strings. They are regarded as 'or' pattern of all elements."
 (defun howm-viewer-call:func (viewer page)
   (funcall (car viewer) page))
 
+(defvar howm-viewer-indicator-format "%%%%%% %s %%%%%%")
+(defun howm-viewer-indicator-gen (fmt &rest args)
+  (format howm-viewer-indicator-format
+          (apply #'format (cons fmt args))))
 (defun howm-viewer-indicator:str (viewer page)
-  (format viewer (howm-page-name page)))
+  (howm-viewer-indicator-gen viewer (howm-page-name page)))
 (defun howm-viewer-indicator:func0 (viewer page)
-  (format "%S %S" viewer page))
+  (howm-viewer-indicator-gen "%S %S" viewer page))
 (defun howm-viewer-indicator:func (viewer page)
-  (format "(%S %S)" (car viewer) page))
+  (let ((func (car viewer))
+        (previewer (cdr viewer)))
+    (if previewer
+        (funcall previewer page)
+      (howm-viewer-indicator-gen "(%S %S)" func page))))
 
 (defadvice action-lock-find-file (around external-viewer (f u) activate)
   (let ((viewer (howm-page-viewer f)))
