@@ -1,7 +1,7 @@
 ;;; howm-common.el --- Wiki-like note-taking tool
-;;; Copyright (c) 2002, 2003, 2004, 2005
+;;; Copyright (c) 2002, 2003, 2004, 2005, 2006
 ;;;   by HIRAOKA Kazuyuki <khi@users.sourceforge.jp>
-;;; $Id: howm-common.el,v 1.63 2005/10/15 15:58:11 hira Exp $
+;;; $Id: howm-common.el,v 1.72 2006/05/15 13:34:32 hira Exp $
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -235,7 +235,10 @@ If PASS-RET-THROUGH is non-nil, RET is unread and nil is returned.
 
 (defun howm-replace-region (beg end val)
   (delete-region beg end)
-  (insert (format "%s" val)))
+  ;; `format' in xemacs doesn't keep text properties.
+  (insert (if (stringp val)
+              val
+            (format "%s" val))))
 
 (put 'howm-edit-read-only-buffer 'lisp-indent-hook 0)
 (defmacro howm-edit-read-only-buffer (&rest body)
@@ -285,11 +288,45 @@ to remove matched lines. REMOVE-FMT must have one %s."
           (delete-region (point) (point-max))))
       (howm-basic-save-buffer))))
 
+(defun howm-get-symbol (soft &rest args)
+  "Return the canonical symbol for a specified name.
+If SOFT is non-nil, return nil when the corresponding symbol does not exist.
+Name of returned symbol is concatenation of ARGS.
+Both strings and symbols are acceptable in ARGS."
+  (funcall (if soft #'intern-soft #'intern)
+           (mapconcat (lambda (s)
+                        (cond ((sequencep s) s)
+                              ((symbolp s) (symbol-name s))
+                              (t (error "Not supported: %S" s))))
+                      args
+                      "")))
+
 ;; snap://Info-mode/elisp#Killing Buffers
 (defun howm-buffer-killed-p (buffer)
   "Return t if BUFFER is killed."
   (not (buffer-name buffer)))
 
+(defun howm-classify (classifier lis &optional reverse)
+  "Classify elements in given list.
+CLASSIFIER is criterion of classification for list LIS.
+If REVERSE is non-nil, order of elements are reversed (faster).
+For example,
+  (howm-classify (lambda (s) (substring s 0 1)) '(\"aaa\" \"abc\" \"xyz\"))
+returns ((\"a\" \"aaa\" \"abc\") (\"x\" \"xyz\"))."
+  (let ((ans nil))
+    (mapc (lambda (x)
+            (let* ((label (funcall classifier x))
+                   (pair (assoc label ans)))
+              (if (null pair)
+                  (setq ans (cons (cons label (list x)) ans))
+                (setcdr pair (cons x (cdr pair))))))
+          lis)
+    (if reverse
+        ans
+      (reverse (mapcar (lambda (pair) (cons (car pair) (reverse (cdr pair))))
+                       ans)))))
+;; (howm-classify (lambda (s) (substring s 0 1)) '("aaa" "abc" "xyz"))
+
 ;; view-in-background
 
 (defvar *howm-view-in-background* nil
@@ -351,7 +388,9 @@ Use `howm-view-in-background' and `howm-view-in-background-p' instead.")
 
 (defun howm-history ()
   (interactive)
-  (howm-menu-open howm-history-file))
+  ;; disable expansion of %schedule etc.
+  (let ((howm-menu-display-rules nil)) ;; dirty
+    (howm-menu-open howm-history-file)))
 
 ;; (defvar howm-history-exclude
 ;;   (let ((strings '("[0-9][0-9][0-9][0-9]" "^[*=] [^ ]")))
@@ -366,21 +405,39 @@ Use `howm-view-in-background' and `howm-view-in-background-p' instead.")
 
 ;;; call process
 
-(defvar howm-call-process-last-command nil)  ;; for debug
+(defvar howm-call-process-last-command nil
+  "List of arguments for last `howm-call-process'.
+This variable exists only for debug. You can reproduce the last call
+with the below code.
+ (apply #'howm-call-process howm-call-process-last-command)")
+
+(put 'howm-with-coding-system 'lisp-indent-hook 1)
+(defmacro howm-with-coding-system (coding-system &rest body)
+  "With CODING-SYSTEM, execute BODY.
+examples:
+ (howm-with-coding-system 'euc-jp-unix ...)  ;; for both read and write
+ (howm-with-coding-system '(utf-8-unix . sjis-unix) ...)  ;; (read . write)
+ (howm-with-coding-system nil ...)  ;; howm-process-coding-system is used."
+  (let ((g (gensym))
+        (cs (or coding-system 'howm-process-coding-system)))
+    `(let* ((,g ,cs)
+            (coding-system-for-read  (or (car-safe ,g) ,g))
+            (coding-system-for-write (or (cdr-safe ,g) ,g)))
+       ,@body)))
+
 (defun howm-call-process (command args
-                                       &optional
-                                       expected-return-value stdin-string)
-;;   (setq howm-call-process-last-command (cons command args))
+                                  &optional expected-return-value stdin-string)
   (setq howm-call-process-last-command (list command args
                                              expected-return-value
                                              stdin-string))
   (with-temp-buffer
-    (let ((r (howm-call-process-here command args stdin-string)))
-      (when (and expected-return-value
-                 (not (= expected-return-value r)))
-        (error "Process returns %s instead of expected %s."
-               r expected-return-value))
-      (howm-buffer-lines))))
+    (howm-with-coding-system nil
+      (let ((r (howm-call-process-here command args stdin-string)))
+        (when (and expected-return-value
+                   (not (= expected-return-value r)))
+          (error "Process returns %s instead of expected %s."
+                 r expected-return-value))
+        (howm-buffer-lines)))))
 
 (defun howm-call-process-here (command args &optional stdin-string)
   (let* ((beg (point))
@@ -388,8 +445,7 @@ Use `howm-view-in-background' and `howm-view-in-background-p' instead.")
                 (insert (or stdin-string ""))
                 (point)))
          (a `(,beg ,end ,command t (t nil) nil ,@args)))
-    (let ((coding-system-for-write howm-process-coding-system)
-          (coding-system-for-read  howm-process-coding-system))
+    (howm-with-coding-system nil
       (apply #'call-process-region a))))
 
 (defun howm-buffer-lines (&optional buf)
@@ -398,7 +454,7 @@ Use `howm-view-in-background' and `howm-view-in-background-p' instead.")
       (set-buffer buf))
     (split-string (buffer-substring (point-min) (point-max)) "\n")))
 
-;;; schedule-interval (clean me)
+;;; schedule-interval & reminder-setting (clean me)
 
 (defvar howm-reminder-schedule-interval nil
   "For internal use. Do not setq this variable.
@@ -417,6 +473,13 @@ after tomorrow'. BODY is evaluated under this setting;
     ,@body))
 (put 'howm-with-schedule-interval 'lisp-indent-hook 1)
 
+(defmacro howm-with-reminder-setting  (&rest body)
+  `(howm-with-schedule-interval
+       (howm-menu-schedule-days-before howm-menu-schedule-days)
+     (let ((howm-todo-menu-types howm-reminder-menu-types))  ;; dirty!
+       ,@body)))
+(put 'howm-with-reminder-setting 'lisp-indent-hook 0)
+
 ;;; xemacs
 
 ;; http://www.bookshelf.jp/cgi-bin/goto.cgi?file=meadow&node=switch%20config
