@@ -1,7 +1,7 @@
 ;;; howm-date.el --- Wiki-like note-taking tool
-;;; Copyright (c) 2002, 2003, 2004, 2005
+;;; Copyright (c) 2002, 2003, 2004, 2005, 2006
 ;;;   by HIRAOKA Kazuyuki <khi@users.sourceforge.jp>
-;;; $Id: howm-date.el,v 1.24 2005/08/29 13:39:28 hira Exp $
+;;; $Id: howm-date.el,v 1.26 2006/01/06 15:35:46 hira Exp $
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -64,19 +64,22 @@
 (defun howm-action-lock-date-prompt (date new pass-through)
   (let* ((dow (howm-datestr-day-of-week date))
          (common-help "+num(shift), yymmdd(set), ~yymmdd(repeat)")
+         (today-help ", .(today)")
          (help (cond ((and new pass-through)
                       common-help)
                      ((and new (not pass-through))
-                      (concat "RET(ok), " common-help ", .(today)"))
+                      (concat "RET(ok), " common-help today-help))
                      ((not new)
-                      (concat "RET(goto), " common-help ", .(today)"))
+                      (concat "RET(list), " common-help today-help))
                      (t
                       (error "Can't happen.")))))
     (format "[%s] %s: " dow help)))
 
 (defun howm-action-lock-date-search (date)
   (howm-set-command 'howm-action-lock-date-search)
-  (howm-search date t))
+  (prog1
+      (howm-search date t)
+    (howm-action-lock-forward-escape)))
 
 (defun howm-search-today ()
   (interactive)
