@@ -1,8 +1,7 @@
 ;;; howm-menu.el --- Wiki-like note-taking tool
-;;; -*- Coding: iso-2022-7bit -*-
-;;; Copyright (c) 2002, 2003, 2004, 2005
+;;; Copyright (c) 2002, 2003, 2004, 2005, 2006
 ;;;   by HIRAOKA Kazuyuki <khi@users.sourceforge.jp>
-;;; $Id: howm-menu.el,v 1.75 2005/07/08 14:11:46 hira Exp $
+;;; $Id: howm-menu.el,v 1.85 2006/02/01 15:50:07 hira Exp $
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -55,8 +54,8 @@
 (defvar howm-menu-list-regexp "^\\(>\\)\\([^|\r\n]*|\\) +\\(.*\\)$"
   "Regexp to find and parse schedule/todo/recent/random list in `howm-menu-mode'.")
 (defvar howm-menu-list-regexp-key-pos 3
-  "Position of target string for action-lock on schedule/todo/recent/random list
-in `howm-menu-mode'. This target is searched when action-lock is invoked.")
+  "Position of target string for action-lock in history buffer.
+This target is searched when action-lock is invoked.")
 (defvar howm-menu-list-regexp-action-pos 1
   "Position of action-lock hilight on schedule/todo/recent/random list
 in `howm-menu-mode'.")
@@ -82,6 +81,7 @@ in `howm-menu-mode'.")
     howm-menu-recent
     howm-menu-random
     howm-menu-search
+    howm-menu-categorized-reminder
     ))
 
 (defvar howm-menu-display-rules
@@ -123,78 +123,11 @@ Regexp R1 is replaced by T1 if T1 is a string.
 
 (defvar howm-menu-action-arg 'howm-menu-action-arg-name)
 
-(defvar howm-menu-command-table-en
-  `(
-    ("[New]" (lambda () (howm-create ,howm-menu-action-arg)))
-    ("[Add]" (lambda () (howm-create-here ,howm-menu-action-arg)))
-    ("[Dup]" howm-dup)
-    ("[Update]" howm-initialize-buffer previous)
-    ("[Regexp]" howm-list-grep)
-    ("[String]" howm-list-grep-fixed)
-    ;;         ("[roma]" howm-list-migemo)
-    ("[Today]" howm-find-today)
-    ("[Yesterday]" howm-find-yesterday)
-    ("[All]" howm-list-all)
-    ("[Recent]" howm-list-recent)
-    ("[Around]" howm-list-around)
-    ("[Schedule]" howm-list-schedule)
-    ("[History]" howm-history)
-    ("[<Title]" howm-keyword-to-kill-ring)
-    ("[<Name]" (lambda () (howm-keyword-to-kill-ring t)))
-    ("[Key>]" howm-insert-keyword previous)
-    ("[Date>]" howm-insert-date previous)
-    ("[DTime>]" howm-insert-dtime previous)
-    ("[Todo]" howm-list-todo)
-    ("[Killall]" howm-kill-all)
-    ("[Force Killall]" (lambda () (interactive) (howm-kill-all t)))
-    ("[Edit Menu]" howm-menu-edit current)
-    ("[Update Menu]" howm-menu-refresh current)
-    ("[Preference]" (lambda () (customize-group 'howm)))
-    ("[Random Walk]" howm-random-walk previous)
-    ))
-
-(defvar howm-menu-command-table-ja
-  `(
-    ;; Japanese characters below
-    ("[$B?75,(B]" (lambda () (howm-create ,howm-menu-action-arg)))
-    ("[$BDI2C(B]" (lambda () (howm-create-here ,howm-menu-action-arg)))
-    ("[$BJ#@=(B]" howm-dup)
-    ("[$B99?7(B]" howm-initialize-buffer previous)
-    ("[$B@55,(B]" howm-list-grep)
-    ("[$B8GDj(B]" howm-list-grep-fixed)
-    ("[roma]" howm-list-migemo)
-    ("[$B:#F|(B]" howm-find-today)
-    ("[$B:rF|(B]" howm-find-yesterday)
-    ("[$B0lMw(B]" howm-list-all)
-    ("[$B:G6a(B]" howm-list-recent)
-    ("[$BA08e(B]" howm-list-around)
-    ("[$BM=Dj(B]" howm-list-schedule)
-    ("[$BMzNr(B]" howm-history)
-    ("[$BBj",(B]" howm-keyword-to-kill-ring)
-    ("[$BL>",(B]" (lambda () (howm-keyword-to-kill-ring t)))
-    ("[$B80"-(B]" howm-insert-keyword previous)
-    ("[$BF|"-(B]" howm-insert-date previous)
-    ("[$B;~"-(B]" howm-insert-dtime previous)
-    ("[Todo]" howm-list-todo)
-    ("[$BA4>C(B]" howm-kill-all)
-    ("[$B6/@)A4>C(B]" (lambda () (interactive) (howm-kill-all t)))
-    ("[menu $BJT=8(B]" howm-menu-edit current)
-    ("[menu $B99?7(B]" howm-menu-refresh current)
-    ("[$B@_Dj(B]" (lambda () (customize-group 'howm)))
-    ("[$B?lJb(B]" howm-random-walk previous)
-    ))
-
 ;;; which is opened as menu?
 
 (defvar howm-menu-keyword-regexp "^%.*%$")
 (defvar howm-menu-top "%menu%")
 
-;;; language
-
-(defvar howm-day-of-week-en "SMTWtFs")
-;; Japanese characters below
-(defvar howm-day-of-week-ja "$BF|7n2P?eLZ6bEZ(B")
-
 ;;; misc.
 
 (defvar howm-menu-toggle-invisible "%|")
@@ -685,51 +618,58 @@ When this is nil, delete-region is used instead, and bug appears.")
 ;;; command
 
 (defun howm-menu-schedule ()
-  (howm-menu-general "schedule"
-                     #'howm-menu-format-reminder
+  (howm-menu-general "schedule" 'schedule
                      (howm-schedule-menu howm-menu-schedule-days
                                          howm-menu-schedule-days-before)))
 
 (defvar howm-menu-todo-show-day-of-week t)
 (defun howm-menu-todo ()
-  (let ((dow-str (cond (howm-menu-todo-show-day-of-week nil)
-                       (t "  "))))
-    (howm-menu-general "todo"
-                       #'(lambda (item)
-                           (if (stringp item)
-                               item
-                             (howm-menu-format-reminder item dow-str t)))
-                       (howm-todo-menu howm-menu-todo-num
-                                       howm-menu-todo-priority
-                                       howm-menu-reminder-separators))))
+  (howm-menu-general "todo" 'todo
+                     (howm-todo-menu howm-menu-todo-num
+                                     howm-menu-todo-priority
+                                     howm-menu-reminder-separators)))
 (defun howm-menu-reminder ()
-  (howm-with-schedule-interval
-      (howm-menu-schedule-days-before howm-menu-schedule-days)
-    (let ((howm-todo-menu-types howm-reminder-menu-types))  ;; dirty!
-      (howm-menu-todo))))
+  (howm-menu-general "reminder" 'todo
+                     (howm-reminder-menu howm-menu-todo-num
+                                         howm-menu-todo-priority
+                                         howm-menu-reminder-separators)))
 
 (defun howm-menu-recent (&optional random)
   (howm-menu-general (if random "random" "recent")
-                     #'(lambda (item)
-                         (let ((info (file-name-sans-extension 
-                                      (howm-view-item-basename item)))
-                               (line (howm-view-item-summary item)))
-                           (howm-menu-list-format info line)))
+                     nil
                      (howm-recent-menu howm-menu-recent-num random)))
 
 (defun howm-menu-random () (howm-menu-recent t))
 
 (defun howm-menu-general (label formatter item-list)
-  (let* ((msg "scanning %s...")
-         (msg-done (concat msg "done")))
-    (message msg label)
-;;     (delete-region (match-beginning 0) (match-end 0))
-    (prog1
-        (mapconcat formatter item-list "\n")
-      (message msg-done label))))
+  "Generate output string for items in howm menu.
+LABEL is only used for message.
+FORMATTER is a function which receives an item and returns an output string
+(without newline).
+FORMATTER can be nil for standard style, 'todo for todo style,
+or 'schedule for schedule style.
+ITEM-LIST is list of items which should be shown."
+  (let ((f (cond ((null formatter) #'howm-menu-format-item)
+                 ((eq 'todo formatter) #'howm-menu-format-todo)
+                 ((eq 'schedule formatter) #'howm-menu-format-reminder)
+                 (t formatter))))
+    (let* ((msg "scanning %s...")
+           (msg-done (concat msg "done")))
+      (message msg label)
+      ;;     (delete-region (match-beginning 0) (match-end 0))
+      (prog1
+          (mapconcat f item-list "\n")
+        (message msg-done label)))))
 
 ;;; schedule/todo
 
+(defun howm-menu-format-todo (item)
+  (if (stringp item) ;; item can be a separator string
+      item
+    (let ((dow-str (cond (howm-menu-todo-show-day-of-week nil)
+                         (t "  "))))
+      (howm-menu-format-reminder item dow-str t))))
+
 (defun howm-menu-format-reminder (item &optional day-of-week-str show-priority)
   (let* ((p (howm-todo-parse item))
          (late (floor (car p)))
@@ -742,7 +682,7 @@ When this is nil, delete-region is used instead, and bug appears.")
                                (howm-todo-priority item))
                      ""))
          (h (format "%s%3s%s" dow-str late priority)))
-    (howm-menu-list-format h (howm-view-item-summary item))))
+    (howm-menu-list-format h (howm-view-item-summary item) item)))
 
 (defun howm-day-of-week-string (&optional day-of-week)
   ;; 0 = Sunday
@@ -789,75 +729,155 @@ When this is nil, delete-region is used instead, and bug appears.")
 
 ;;; common
 
-(defun howm-menu-list-format (info line)
-  (format howm-menu-list-format info line))
+(defun howm-menu-list-put-item (text item)
+  ;; put it to whole text, because I don't assume "> ..." format here.
+  (put-text-property 0 (length text) 'howm-menu-list-item item text))
+(defun howm-menu-list-get-item (&optional text)
+  (get-text-property (if text 0 (point)) 'howm-menu-list-item text))
+(defun howm-menu-list-getput-item (from-text to-text)
+  (howm-menu-list-put-item to-text
+                           (howm-menu-list-get-item from-text)))
+
+(defun howm-menu-list-action (&optional keyword)
+  (let ((item (howm-menu-list-get-item keyword)))
+    (cond (item (howm-view-open-item item)) ;; schedule, todo, etc.
+          (keyword (howm-keyword-search keyword)) ;; history
+          (t (error "Target is not specified."))))) ;; can't happen
+
+(defun howm-menu-format-item (item)
+  (let* ((info (file-name-sans-extension (howm-view-item-basename item)))
+         (line (howm-view-item-summary item)))
+    (howm-menu-list-format info line item)))
+
+(defun howm-menu-list-format (info line item)
+  (let ((s (format howm-menu-list-format info line)))
+    (howm-menu-list-put-item s item)
+    s))
 
 (defun howm-menu-list-rules ()
-  (list (howm-action-lock-search howm-menu-list-regexp
-                                 howm-menu-list-regexp-key-pos
-                                 howm-menu-list-regexp-action-pos
-                                 nil
-                                 t)))
+  (list (action-lock-general #'howm-menu-list-action
+                             howm-menu-list-regexp
+                             howm-menu-list-regexp-key-pos
+                             howm-menu-list-regexp-action-pos)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; embed search result
 
 (defun howm-menu-search (key)
-  (let* ((folder (howm-make-folder:files (howm-search-path)))
-         (is (howm-view-search-folder-items key folder nil 'fixed)))
-    (mapconcat (lambda (item)
-                 (howm-menu-list-format (howm-view-item-basename item)
-                                        (howm-item-summary item)))
-               is
-               "\n")))
+  (howm-menu-general "menu-search"
+                     nil
+                     (howm-view-search-folder-items key (howm-folder)
+                                                    nil 'fixed)))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; categorized todo-list
+
+;; Experimental [2006-01-16]
+
+(defun howm-menu-classified-reminder (classifier &optional comparer
+                                                 title-format)
+  "Generate string of classified reminder-list.
+CLASSIFIER is a function which receives an item and answers its class.
+Class can be an arbitrary lisp object.
+When class is nil, corresponding item is not shown in this list.
+COMPARER is a function which receives two keys and answer t or nil.
+It is used for sorting of keys.
+TITLE-FORMAT is a format string for class title."
+  (let* ((a (howm-classify classifier
+                           (howm-reminder-menu nil
+                                               howm-menu-todo-priority
+                                               nil)))
+         ;; key 'nil' is skipped.
+         (keys (remove nil (mapcar #'car a)))
+         (tform (concat (or title-format "--%s--") "\n")))
+    (when comparer
+      (setq keys (sort keys comparer)))
+    (mapconcat (lambda (k)
+                 (let* ((item-list (howm-first-n (cdr (assoc k a))
+                                                 howm-menu-todo-num))
+                        (is (howm-with-reminder-setting
+                              (howm-todo-insert-separators
+                               item-list
+                               howm-menu-reminder-separators))))
+                   (concat (format tform k)
+                           (howm-menu-general (format "reminder(%s)" k) 'todo
+                                              is))))
+               keys "\n")))
+
+(defun howm-menu-categorized-reminder (categories &optional title-format
+                                                  erase-p omit-misc-p)
+  "Generate string of categorized reminder-list.
+
+Write %here%(howm-menu-categorized-reminder (\"foo\" \"bar\" \"baz\"))
+to show categorized list in menu. (You don't need quote(')
+before the above list; arguments are not evaluated in %here%
+because I don't have enough courage to call eval.)
+
+If you like to erase category label from summary string, try
+%here%(howm-menu-categorized-reminder (\"foo\" \"bar\" \"baz\") nil t)
+instead.
+
+If you don't like misc. category, try
+%here%(howm-menu-categorized-reminder (\"foo\" \"bar\" \"baz\") nil nil t)."
+  ;; Using categories, matcher, etc. in lambda is bad indeed
+  ;; because of dynamic binding.
+  (let* ((matcher (lambda (cat str item)
+                    (and (string-match (regexp-quote cat) str)
+                         (progn
+                           (when erase-p
+                             (howm-item-set-summary item
+                                                    (replace-match "" nil nil
+                                                                   str)))
+                           t))))
+         (classifier (lambda (item)
+                       (let ((s (howm-item-summary item)))
+                         (or (howm-cl-find-if (lambda (c)
+                                                (funcall matcher c s item))
+                                              categories)
+                             (if omit-misc-p nil "misc.")))))
+         (pos (lambda (c) (or (howm-cl-position c categories) howm-infinity)))
+         (comparer (lambda (a b) (< (funcall pos a) (funcall pos b)))))
+    (howm-menu-classified-reminder classifier comparer title-format)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; generate initial menu
 
-(defun howm-menu-initialize-skel (no-chance)
-  (let ((fd (howm-menu-skel)))
-    (if (and fd (not no-chance))
-        (progn
-          (howm-menu-copy-skel fd)
-          (howm-view-kill-buffer)
-          (howm-menu nil t))
-      (error "Can't find menu-file."))))
+(defun howm-menu-initialize-skel (&optional dummy)
+  (let ((menu-name (howm-get-symbol nil "howm-menu-" howm-menu-lang)))
+    (require menu-name)
+    (howm-menu-copy-skel (symbol-value menu-name))
+    (howm-view-kill-buffer)
+    (howm-menu nil t)))
 
-(defun howm-menu-skel ()
-  (let* ((v (if (eq howm-menu-lang 'ja)
-                'howm-ja-dir
-              'howm-en-dir))
-         (d (and (boundp v) (eval v)))
-         (f "0000-00-00-000000.howm"))
-    (if (and d (file-exists-p (expand-file-name f d)))
-        (list f d)
-      nil)))
-
-(defun howm-menu-copy-skel (file-dir &optional menu-file)
-  (let* ((f (car file-dir))
-         (d (second file-dir))
-         (s (expand-file-name f d))
-         (menu-file (or menu-file 
-                        (expand-file-name f howm-directory))))
+(defun howm-menu-copy-skel (contents)
+  (let ((menu-file (or howm-menu-file
+                       (expand-file-name "0000-00-00-000000.howm"
+                                         howm-directory))))
     (if (file-exists-p menu-file)
+        ;; I have no courage to erase existing file.
         (progn
           (setq howm-menu-file menu-file)
           (message "Assume %s as menu file." menu-file))
-      (copy-file s menu-file))))
+      (progn
+        (find-file menu-file)
+        (insert contents)
+        (save-buffer)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; switch language
 
-(defun howm-menu-lang-assoc (lang)
-  (let ((a `((default . (,howm-menu-command-table-en
-                         ,howm-day-of-week-en))
-             (ja . (,howm-menu-command-table-ja
-                    ,howm-day-of-week-ja)))))
-    (cdr (or (assoc lang a)
-             (assoc 'default a)))))
+(defun howm-require-lang (&optional lang)
+  (require (howm-get-symbol nil "howm-lang-" (or lang howm-menu-lang))))
+
+(defun howm-lang-ref (var)
+  (let ((lang howm-menu-lang))
+    (howm-require-lang lang)
+    ;; For backward compatibility, I use howm-day-of-week-en
+    ;; rather than howm-day-of-week:en.
+    (symbol-value (howm-get-symbol t var "-" lang))))
 
 (defun howm-menu-command-table-raw ()
-  (car (howm-menu-lang-assoc howm-menu-lang)))
+  (howm-lang-ref "howm-menu-command-table"))
 
 (defun howm-menu-command-table ()
   (append howm-menu-command-table-common
@@ -865,7 +885,7 @@ When this is nil, delete-region is used instead, and bug appears.")
                   (howm-menu-command-table-raw))))
 
 (defun howm-day-of-week ()
-  (second (howm-menu-lang-assoc howm-menu-lang)))
+  (howm-lang-ref "howm-day-of-week"))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; misc.
