@@ -1,7 +1,7 @@
 ;;; howm-misc.el --- Wiki-like note-taking tool
-;;; Copyright (c) 2002, 2003, 2004, 2005
+;;; Copyright (c) 2002, 2003, 2004, 2005, 2006
 ;;;   by HIRAOKA Kazuyuki <khi@users.sourceforge.jp>
-;;; $Id: howm-misc.el,v 1.65 2005/08/11 15:10:18 hira Exp $
+;;; $Id: howm-misc.el,v 1.72 2006/05/01 15:07:05 hira Exp $
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -380,11 +380,18 @@ and replace a sub-expression, e.g.
   (string= (substring (buffer-name buf) 0 1) " "))
 
 (defun howm-buffer-p (&optional buf)
-  (with-current-buffer (or buf (current-buffer))
-    (or howm-mode
-        (member major-mode
-                '(howm-view-summary-mode
-                  howm-view-contents-mode)))))
+  (let* ((indep-dirs (cons nil *howm-independent-directories*))
+         (keyword-bufs (mapcar
+                        (lambda (d)
+                          (let ((default-directory (or d default-directory)))
+                            (howm-keyword-buffer)))
+                        indep-dirs)))
+    (with-current-buffer (or buf (current-buffer))
+      (or howm-mode
+          (member major-mode
+                  '(howm-view-summary-mode
+                    howm-view-contents-mode))
+          (member buf keyword-bufs)))))
 
 (defun howm-mode-add-font-lock ()
   (when howm-use-color
@@ -542,22 +549,95 @@ and replace a sub-expression, e.g.
       (search-backward k nil t))
     (sit-for howm-random-walk-wait)
     (howm-keyword-search k)))
-;;     (howm-random-walk-invoke)))
 
-;; (defun howm-random-walk-invoke ()
-;;   (setq howm-random-walk-buf (current-buffer)
-;;         howm-random-walk-ro buffer-read-only)
-;;   (setq buffer-read-only t)
-;;   (unwind-protect
-;;       (when (howm-random-walk-invoke-p)
-;;         (action-lock-invoke))
-;;     (when (buffer-live-p howm-random-walk-buf)
-;;       (with-current-buffer howm-random-walk-buf
-;;         (setq buffer-read-only howm-random-walk-ro)))))
+;; named note
 
-;; (defun howm-random-walk-invoke-p ()
-;;   (let ((search-action (second (howm-action-lock-search "dummy" 0))))
-;;     (equal (action-lock-get-action) search-action)))
+(defun howm-open-named-file ()
+  "Ask a file name and open it as howm note if it is under howm directory."
+  (interactive)
+  (let* ((item-dir (lambda (item) (file-name-directory (howm-item-name item))))
+         (dir (cond ((eq major-mode 'howm-view-summary-mode)
+                     (funcall item-dir (howm-view-summary-current-item)))
+                    ((eq major-mode 'howm-view-contents-mode)
+                     (funcall item-dir (howm-view-contents-current-item)))
+                    (t
+                     howm-directory)))
+         (fname (read-file-name "Howm file name: " dir)))
+    (find-file fname)
+    (if (file-exists-p fname)
+        (howm-set-mode)
+      (progn
+        (howm-insert-template "")
+        (howm-create-finish)))))
+
+;; imitation of remember.el
+;; http://www.emacswiki.org/cgi-bin/emacs-en/RememberMode
+
+;; shamelessly copied from http://newartisans.com/johnw/Emacs/remember.el
+;; (I cannot browse http://sacha.free.net.ph/notebook/emacs/dev today.)
+
+(defvar howm-remember-wconf nil
+  "for internal use")
+(defvar howm-remember-buffer-name "*howm-remember*")
+(defvar howm-remember-mode-hook nil)
+(let ((m (make-sparse-keymap)))
+  (define-key m "\C-c\C-c" 'howm-remember-submit)
+  (define-key m "\C-c\C-k" 'howm-remember-discard)
+  (defvar howm-remember-mode-map m))
+(defvar howm-remember-first-line-to-title nil)
+
+(defun howm-remember ()
+  "Add text to new note in howm."
+  (interactive)
+  (setq howm-remember-wconf (current-window-configuration))
+  (switch-to-buffer-other-window (get-buffer-create howm-remember-buffer-name))
+  (howm-remember-mode)
+  (apply #'message
+         `("Remember (%s) or discard (%s)."
+           ,@(mapcar (lambda (f)
+                       (key-description
+                        (where-is-internal f howm-remember-mode-map t)))
+                     '(howm-remember-submit howm-remember-discard)))))
+
+(defun howm-remember-mode ()
+  "Major mode for `howm-remember'.
+
+\\{howm-remember-mode-map}"
+  (interactive)
+  (kill-all-local-variables)
+  (text-mode)
+  (use-local-map howm-remember-mode-map)
+  (setq major-mode 'howm-remember-mode
+        mode-name "HowmRemember")
+  (run-hooks 'howm-remember-mode-hook))
+
+(defun howm-remember-submit ()
+  (interactive)
+  (save-excursion
+    (let* ((title (howm-remember-get-title)) ;; has side effect
+           (s (buffer-substring-no-properties (point-min) (point-max))))
+      (set-window-configuration howm-remember-wconf)
+      (howm-create-file-with-title title)
+      (insert s "\n")
+      (save-buffer)
+      (kill-buffer (current-buffer))))
+  (howm-remember-discard))
+
+(defun howm-remember-get-title ()
+  (if (not howm-remember-first-line-to-title)
+      ""
+    (progn
+      (goto-char (point-min))
+      (prog1
+          (buffer-substring-no-properties (point-min)
+                                          (line-end-position))
+        (forward-line 1)
+        (delete-region (point-min) (point))))))
+
+(defun howm-remember-discard ()
+  (interactive)
+  (kill-buffer (current-buffer))
+  (set-window-configuration howm-remember-wconf))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Fellowship
