@@ -1,7 +1,7 @@
 ;;; howm-mode.el --- Wiki-like note-taking tool
-;;; Copyright (c) 2002, 2003, 2004, 2005
+;;; Copyright (c) 2002, 2003, 2004, 2005, 2006
 ;;;   by HIRAOKA Kazuyuki <khi@users.sourceforge.jp>
-;;; $Id: howm-mode.el,v 1.285 2005/11/04 13:04:33 hira Exp $
+;;; $Id: howm-mode.el,v 1.290 2006/03/22 13:29:25 hira Exp $
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -64,7 +64,7 @@ If it is a function, it is called to get template string with the argument <n>."
 
 (defvar howm-default-key-table
   '(
-    ;; ("key"  func list-mode-p global-p)
+    ;; ("key" func list-mode-p global-p)
     ("r" howm-refresh)
     ("l" howm-list-recent t t)
     ("a" howm-list-all t t)
@@ -95,6 +95,7 @@ If it is a function, it is called to get template string with the argument <n>."
     ("C" howm-create-here nil t)
     ("I" howm-create-interactively nil t)
     ("w" howm-random-walk nil t)
+    ("M" howm-open-named-file t t)
     )
   "List of (key function list-mode-p).
 `howm-prefix' + this key is real stroke.
@@ -104,6 +105,16 @@ same key is also available in view mode."
 
 (defvar howm-migemo-client nil)
 ;; (defvar howm-migemo-client "migemo-client")
+(defvar howm-migemo-client-option nil
+  "List of option for migemo-client.
+e.g. (\"-H\" \"::1\")")
+
+;; (defvar howm-migemo-client nil
+;;   "Command name (string) of migemo-client. Nil means default command.
+;; Example:
+;;   (setq howm-migemo-client \"my-migemo-client-command\")
+;;   (setq howm-migemo-client '(\"migemo-client\" \"-H\" \"::1\")) ;; options
+;;   (setq howm-migemo-client '(nil \"-H\" \"::1\")) ;; same as the above")
 
 ;;; --- level 2 ---
 
@@ -282,6 +293,7 @@ key	binding
               howm-default-key-table)
     ))
 
+;; emacs20's easy-mmode-define-minor-mode can't have body. sigh...
 (add-hook 'howm-mode-on-hook 'howm-initialize-buffer)
 (add-hook 'howm-mode-off-hook 'howm-restore-buffer)
 
@@ -482,7 +494,6 @@ key	binding
           (howm-search g-reg nil e-reg)
         (message "No response from migemo-client.")))))
 
-
 (defun howm-migemo-get-pattern (roma type)
   (when (and (null howm-migemo-client) (not howm-view-use-grep))
     (require 'migemo))
@@ -490,7 +501,26 @@ key	binding
       (howm-funcall-if-defined (migemo-get-pattern roma))
 ;;       (migemo-get-pattern roma)
     (car (howm-call-process (or howm-migemo-client "migemo-client")
-                            `("-t" ,type ,roma) 0))))
+                            `(,@howm-migemo-client-option "-t" ,type ,roma)
+                            0))))
+
+;; (defun howm-migemo-get-pattern (roma type)
+;;   (when (and (null (howm-migemo-client)) (not howm-view-use-grep))
+;;     (require 'migemo))
+;;   (if (and (featurep 'migemo) (string= type "emacs"))
+;;       (howm-funcall-if-defined (migemo-get-pattern roma))
+;; ;;       (migemo-get-pattern roma)
+;;     (car (howm-call-process (howm-migemo-client)
+;;                             `(,@(howm-migemo-client-option) "-t" ,type ,roma)
+;;                             0))))
+
+;; (defun howm-migemo-client ()
+;;   (if (stringp howm-migemo-client)
+;;       howm-migemo-client
+;;     (or (car howm-migemo-client) "migemo-client")))
+
+;; (defun howm-migemo-client-option ()
+;;   (cdr-safe howm-migemo-client))
 
 (defun howm-list-normalize (&optional keyword comefrom-regexp no-list-title)
   "Sort displayed items in the standard order.
@@ -679,8 +709,11 @@ We need entire-match in order to
         (undo-boundary)
         (delete-region p end))
       (funcall insert-f use-file))
-    (howm-set-mode)
-    (run-hooks 'howm-create-hook)))
+    (howm-create-finish)))
+
+(defun howm-create-finish ()
+  (howm-set-mode)
+  (run-hooks 'howm-create-hook))
 
 (defun howm-create-newline ()
   (widen)
