@@ -1,7 +1,7 @@
 ;;; howm-reminder.el --- Wiki-like note-taking tool
-;;; Copyright (c) 2002, 2003, 2004, 2005
+;;; Copyright (c) 2002, 2003, 2004, 2005, 2006
 ;;;   by HIRAOKA Kazuyuki <khi@users.sourceforge.jp>
-;;; $Id: howm-reminder.el,v 1.54 2005/07/27 11:40:55 hira Exp $
+;;; $Id: howm-reminder.el,v 1.58 2006/01/23 14:40:00 hira Exp $
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -43,14 +43,14 @@
 (defvar howm-todo-priority-deadline-laziness 7)
 (defvar howm-todo-priority-deadline-init -2)
 (defvar howm-todo-priority-schedule-laziness 1)
-(defvar howm-todo-priority-normal-bottom   (- howm-huge)) 
-(defvar howm-todo-priority-todo-bottom     (- howm-huge)) 
-(defvar howm-todo-priority-defer-bottom    (- howm-huge)) 
-(defvar howm-todo-priority-deadline-bottom (- howm-huge)) 
-(defvar howm-todo-priority-schedule-bottom (- howm-huge)) 
-(defvar howm-todo-priority-deadline-top    howm-huge)              
+(defvar howm-todo-priority-normal-bottom   (- howm-huge))
+(defvar howm-todo-priority-todo-bottom     (- howm-huge))
+(defvar howm-todo-priority-defer-bottom    (- howm-huge))
+(defvar howm-todo-priority-deadline-bottom (- howm-huge))
+(defvar howm-todo-priority-schedule-bottom (- howm-huge))
+(defvar howm-todo-priority-deadline-top    howm-huge)
 (defvar howm-todo-priority-schedule-top    howm-huge)
-(defvar howm-todo-priority-unknown-top     howm-huge+)             
+(defvar howm-todo-priority-unknown-top     howm-huge+)
 
 (defvar howm-action-lock-reminder-done-default nil)
 
@@ -403,6 +403,12 @@ This value is passed to `format-time-string', and the result must be a regexp."
     (howm-todo-sort-by-priority)))
 
 (defun howm-todo-menu (n limit-priority separators)
+  "Find top N todo items, or all todo items if N is nil.
+Returned value is a sorted list of items (see `howm-make-item').
+Items whose priority is worse than LIMIT-PRIORITY are eliminated.
+Separator strings are inserted to the returned list according to
+the rule given as SEPARATORS.
+See docstring of the variable `howm-menu-reminder-separators' for details."
   (let* ((cutted (howm-cl-remove-if (lambda (item)
                                       (< (howm-todo-priority item)
                                          limit-priority))
@@ -410,12 +416,14 @@ This value is passed to `format-time-string', and the result must be a regexp."
          (sorted (howm-sort #'howm-todo-priority-ext
                             #'howm-todo-priority-ext-gt
                             cutted)))
-    (if (null sorted)
-        nil
-      (let ((head (if (< (length sorted) n)
-                      sorted
-                    (howm-cl-subseq sorted 0 n))))
-        (howm-todo-insert-separators head separators)))))
+    (howm-todo-insert-separators (if n (howm-first-n sorted n) sorted)
+                                 separators)))
+
+(defun howm-reminder-menu (n limit-priority separators)
+  (howm-with-reminder-setting
+    (howm-todo-menu howm-menu-todo-num
+                    limit-priority
+                    separators)))
 
 (defun howm-todo-insert-separators (item-list separators)
   (let ((is (mapcar (lambda (item) (cons (howm-todo-priority item) item))
@@ -668,27 +676,42 @@ When D is t, the beginning of today is encoded."
                       menu-date-rule)))
     (list reminder-rule date-rule)))
 
+(defvar howm-action-lock-forward-wconf nil
+  "for internal use")
+(defun howm-action-lock-forward-escape ()
+  (setq howm-action-lock-forward-wconf
+        (current-window-configuration)))
+(put 'howm-action-lock-forward-block 'lisp-indent-hook 0)
+(defmacro howm-action-lock-forward-block (&rest body)
+  `(prog2
+       (setq howm-action-lock-forward-wconf nil)
+       (progn
+         ,@body)
+     (when howm-action-lock-forward-wconf
+       (set-window-configuration howm-action-lock-forward-wconf))))
+
 (defun howm-action-lock-forward (form-pos)
-  (let* ((cursor-pos (point))
-         (form-reg (howm-line-tail-regexp form-pos))
-         (cursor-reg (howm-line-tail-regexp cursor-pos)))
-    (let* ((mt (buffer-modified-tick))
-           (original-tail (buffer-substring-no-properties form-pos
-                                                          (line-end-position)))
-           (modified-tail (howm-action-lock-forward-invoke form-reg
-                                                           cursor-reg))
-           (untouched-p (= mt (buffer-modified-tick))))
-      ;; Current-buffer may be already updated according to
-      ;; howm-menu-refresh-after-save because save-buffer in
-      ;; howm-action-lock-forward-invoke can run howm-after-save-hook.
-      ;; We have to exclude such cases.
-      (when (and untouched-p
-                 (not (string= original-tail modified-tail)))
-        (let ((buffer-read-only nil))
-          (delete-region form-pos (line-end-position))
-          (insert modified-tail)))
-      (goto-char cursor-pos)
-      (howm-action-lock-forward-update))))
+  (howm-action-lock-forward-block
+    (let* ((cursor-pos (point))
+           (form-reg (howm-line-tail-regexp form-pos))
+           (cursor-reg (howm-line-tail-regexp cursor-pos)))
+      (let* ((mt (buffer-modified-tick))
+             (original-tail (buffer-substring form-pos (line-end-position)))
+             (modified-tail (howm-action-lock-forward-invoke form-reg
+                                                             cursor-reg))
+             (untouched-p (= mt (buffer-modified-tick))))
+        ;; Current-buffer may be already updated according to
+        ;; howm-menu-refresh-after-save because save-buffer in
+        ;; howm-action-lock-forward-invoke can run howm-after-save-hook.
+        ;; We have to exclude such cases.
+        (when (and untouched-p
+                   (not (string= original-tail modified-tail)))
+          (let ((buffer-read-only nil))
+            (howm-menu-list-getput-item original-tail modified-tail)
+            (delete-region form-pos (line-end-position))
+            (insert modified-tail)))
+        (goto-char cursor-pos)
+        (howm-action-lock-forward-update)))))
 
 (defun howm-line-tail-regexp (pos)
   (concat (regexp-quote (buffer-substring-no-properties pos
@@ -739,8 +762,7 @@ When D is t, the beginning of today is encoded."
 (defun howm-action-lock-forward-open ()
   (cond ((eq major-mode 'howm-menu-mode)
          (progn
-           (beginning-of-line)
-           (action-lock-magic-return)
+           (howm-menu-list-action)
            (when (eq major-mode 'howm-view-summary-mode)
              (error "Several candidates."))))
         ((eq major-mode 'howm-view-summary-mode)
@@ -773,6 +795,8 @@ or TODO is t."
   (let* ((schedule-menu (eq schedule t))
          (todo-menu (eq todo t))
          (reminder-menu (or schedule-menu todo-menu)))
+    ;; Don't modify howm-reminder-marks.
+    ;; Otherwise, defcustom will be confused for howm-reminder-menu-types, etc.
     (mapcar* (lambda (var flag) (howm-modify-reminder-types var letter flag))
              '(howm-reminder-types
                howm-schedule-types howm-todo-types
@@ -798,26 +822,25 @@ Example:
                   removed)))
       (set var (format "[%s]" new)))))
 
-;; I will comment out below example in future. [2005-05-17]
-
+;; (example)
 ;; If you write like below in your memo, it will appear
 ;; under today's schedule in reminder list.
 ;; The date "2004-11-01" is dummy and "0" means the position "today - 0".
 ;;   [2004-11-01]_0 ========================
-(defun howm-todo-priority-separator (late lazy item)
-  (- howm-huge (or lazy 0) -1))
-(defface howm-reminder-separator-face
-  ;; invisible :p
-  '((((class color) (background light)) (:foreground "white"))
-    (((class color) (background dark)) (:foreground "black"))
-    (t ()))
-  "Face for `howm-list-reminder'. This is obsolete and will be removed in future."
-  :group 'howm-faces)
-(defvar howm-reminder-separator-face 'howm-reminder-separator-face)
-(defvar howm-reminder-separator-type "_")
-(howm-define-reminder howm-reminder-separator-type
-                      #'howm-todo-priority-separator
-                      'howm-reminder-separator-face t t)
+;; (defun howm-todo-priority-separator (late lazy item)
+;;   (- howm-huge (or lazy 0) -1))
+;; (defface howm-reminder-separator-face
+;;   ;; invisible :p
+;;   '((((class color) (background light)) (:foreground "white"))
+;;     (((class color) (background dark)) (:foreground "black"))
+;;     (t ()))
+;;   "Face for `howm-list-reminder'. This is obsolete and will be removed in future."
+;;   :group 'howm-faces)
+;; (defvar howm-reminder-separator-face 'howm-reminder-separator-face)
+;; (defvar howm-reminder-separator-type "_")
+;; (howm-define-reminder howm-reminder-separator-type
+;;                       #'howm-todo-priority-separator
+;;                       'howm-reminder-separator-face t t)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; provide
