@@ -1,7 +1,7 @@
 ;;; howm-vars.el --- Wiki-like note-taking tool
-;;; Copyright (c) 2005
+;;; Copyright (c) 2005, 2006
 ;;;   by HIRAOKA Kazuyuki <khi@users.sourceforge.jp>
-;;; $Id: howm-vars.el,v 1.27 2005/11/04 13:04:33 hira Exp $
+;;; $Id: howm-vars.el,v 1.31 2006/05/15 13:34:32 hira Exp $
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -229,8 +229,17 @@ A file is excluded iff this regexp matches with all the relative paths."
 
 (defun howm-custom-reminder-get-types (symbol)
   (let ((reg (default-value symbol))
-        (types (split-string howm-reminder-marks "")))
-    (howm-cl-remove-if-not (lambda (ty) (string-match reg ty)) types)))
+        (default-types (split-string howm-reminder-marks "")))
+    ;; return list of types for standard cases
+    ;; and string itself for nonstandard cases
+    (if (not (string-match "^\\[\\(.*\\)\\]" reg))
+        reg
+      (let ((types (split-string (match-string-no-properties 1 reg) "")))
+        (if (howm-cl-find-if-not (lambda (x) (member x default-types))
+                                 types)
+            reg
+          (howm-cl-remove-if-not (lambda (x) (member x types))
+                                 default-types))))))
 (defun howm-custom-reminder-set-types (symbol types)
   (when (listp types)
     (setq types (apply #'concat `("[" ,@types "]"))))
@@ -324,10 +333,10 @@ Be careful that you cannot undo the result of action-lock after kill-buffer."
 (defvar howm-huge 77777)
 (defvar howm-huge+ 88888)
 
-(defcustom howm-menu-todo-priority (- howm-huge-)
+(defcustom howm-menu-todo-priority (- howm-huge+)
   "*Limit priority for elimination of reminders in menu."
-  :type `(radio (const :tag "Hide sleeping reminders" ,(- howm-huge-))
-                (const :tag "Show sleeping reminders",(- howm-huge+))
+  :type `(radio (const :tag "Show sleeping reminders",(- howm-huge+))
+                (const :tag "Hide sleeping reminders" ,(- howm-huge-))
                 number)
   :group 'howm-menu-reminder)
 
@@ -917,11 +926,6 @@ If the value is t, it means 'always'."
   :type 'hook
   :group 'howm-hooks)
 
-(defcustom howm-mode-off-hook nil
-  "Hook run when `howm-mode' is turned off."
-  :type 'hook
-  :group 'howm-hooks)
-
 (defcustom howm-view-open-hook nil
   "Hook run when open a note from summary/contents buffer."
   :type 'hook
@@ -1013,11 +1017,16 @@ This option is prepared for `howm-menu-reminder'."
 
 (defcustom howm-process-coding-system nil
   "*Default coding system for grep command in howm.
+If the value is a symbol, it is used for both read and write.
+If the value is a cons pair, its car and cdr are used for read and write,
+respectively.
 
 Example:
-(setq howm-process-coding-system 'euc-japan-unix)"
+ (setq howm-process-coding-system 'euc-japan-unix)
+ (setq howm-process-coding-system '(utf-8-unix . sjis-unix))"
   :type '(radio (const :tag "Off" nil)
-                symbol)
+                coding-system
+                (cons coding-system coding-system))
 ;;   :group 'howm-grep
   :group 'howm-experimental
   )
