@@ -3,7 +3,8 @@
 ;;; @configure_input@
 
 (provide 'howm-version)
-(defconst howm-version "1.3.2")
-(defconst howm-en-dir "/usr/local/share/howm/en")
-(defconst howm-ja-dir "/usr/local/share/howm/ja")
+(defconst howm-version "1.3.4")
+(defconst howm-en-dir "~endir~")
+(defconst howm-ja-dir "~jadir~")
+
 ;;; howm-version.el ends here
