@@ -1,7 +1,7 @@
 ;;; howm-view.el --- Wiki-like note-taking tool
-;;; Copyright (c) 2002, 2003, 2004, 2005
+;;; Copyright (c) 2002, 2003, 2004, 2005, 2006
 ;;;   by HIRAOKA Kazuyuki <khi@users.sourceforge.jp>
-;;; $Id: howm-view.el,v 1.204 2005/10/26 15:38:00 hira Exp $
+;;; $Id: howm-view.el,v 1.209 2006/03/22 13:29:25 hira Exp $
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -117,7 +117,9 @@
 (defalias 'howm-view-persistent-p  #'riffle-persistent-p)  
 (defalias 'howm-view-kill-buffer   #'riffle-kill-buffer)   
 (defalias 'howm-view-set-place     #'riffle-set-place)     
-(defalias 'howm-view-summary-to-contents 'riffle-summary-to-contents)
+(defalias 'howm-view-summary-current-item  #'riffle-summary-current-item)
+(defalias 'howm-view-contents-current-item #'riffle-contents-current-item)
+(defalias 'howm-view-summary-to-contents   #'riffle-summary-to-contents)
 (defalias 'howm-view-restore-window-configuration #'riffle-restore-window-configuration)
 
 ;; for howmoney.el
@@ -380,8 +382,7 @@ key	binding
 (defvar *howm-view-item-privilege* nil) ;; dirty
 
 (defun howm-view-contents-open-sub (&optional kill)
-  (let* ((n (riffle-contents-item-number (point)))
-         (item (nth n (riffle-item-list)))
+  (let* ((item (riffle-contents-current-item))
          (page (howm-item-page item))
          (offset (howm-view-item-offset item))
          (pos (- (point) offset))
@@ -394,15 +395,26 @@ key	binding
     (run-hooks 'howm-view-before-open-hook)
     (if viewer
         (howm-view-call-external-viewer viewer page)
-      (progn
-        (howm-page-open page)
-        (howm-view-set-mark-command)
-        (when (or (< pos (point-min)) (<= (point-max) pos))
-          (widen))
-        (goto-char pos)
-        (recenter howm-view-open-recenter)))
+      (howm-view-open-item item
+                           (lambda ()
+                             (when (or (< pos (point-min)) (<= (point-max) pos))
+                               (widen))
+                             (goto-char pos))
+                           t))
     (run-hooks 'howm-view-open-hook)))
 
+(defun howm-view-open-item (item &optional position-setter merely)
+  (howm-page-open (howm-item-page item))
+  (howm-view-set-mark-command)
+  (if position-setter
+      (funcall position-setter)
+    (howm-view-set-place (howm-item-place item)))
+  (recenter howm-view-open-recenter)
+  (when (not merely)
+    (howm-view-open-postprocess)))
+(defun howm-view-open-postprocess ()
+  (run-hooks 'howm-view-open-hook))
+
 (defvar howm-view-previous-section-page nil "For internal use")
 (defvar howm-view-previous-section-beg nil "For internal use")
 (defvar howm-view-previous-section-end nil "For internal use")
@@ -479,9 +491,7 @@ But I'm not sure for multi-byte characters on other versions of emacsen."
     (list (point-min) (point-max))))
 
 (defun howm-view-contents-indicator (viewer fname)
-  (insert "%%% "
-          (howm-viewer-indicator viewer fname)
-          " %%%"))
+  (insert (howm-viewer-indicator viewer fname)))
 
 (defun howm-view-paragraph-region (&optional include-following-blank-p)
   (let ((b (save-excursion
@@ -821,7 +831,25 @@ This feature does not work when `howm-view-search-in-result-correctly' is nil."
 ;;   :group 'howm-efficiency
   :group 'howm-experimental)
 
+(defcustom howm-view-list-title-type 1
+  "*Type of showing title in summary buffer.
+Value 1 means \"show title instead of summary\".
+Value 2 means \"show title before summary\".
+You may want to set `howm-view-summary-format' to be \"\" if you never need
+to see file names."
+  :type '(radio (const :tag "title instead of summary"
+                       1)
+                (const :tag "title before summary"
+                       2))
+  :group 'howm-experimental)
+
 (defun howm-view-list-title (title-regexp)
+  (if (= howm-view-list-title-type 1)
+      (howm-view-list-title1 title-regexp)
+    (howm-view-list-title2 title-regexp)))
+
+(defun howm-view-list-title1 (title-regexp)
+  "Show title instead of summary."
   (let* ((folder (howm-make-folder-from-items (howm-view-item-list)))
          (items (howm-view-search-folder-items title-regexp folder))
          (kw *howm-view-font-lock-keywords*))
@@ -851,10 +879,22 @@ This feature does not work when `howm-view-search-in-result-correctly' is nil."
         (let ((*howm-view-font-lock-keywords* kw)) ;; dirty!
           (howm-view-summary-rebuild all-items))))))
 
+(defun howm-view-list-title2 (title-regexp)
+  "Show title before summary."
+  (let ((item-list (howm-view-item-list)))
+    (mapc (lambda (item)
+            (let ((orig (howm-item-summary item))
+                  (titles (howm-item-titles item)))
+              (when titles
+                (howm-item-set-summary item
+                                       (format "%-13s | %s"
+                                               (car titles) orig)))))
+          item-list)
+    (howm-view-summary-rebuild item-list)))
+
 ;;; detect items in same paragraph (= entry = memo. sorry for inconsistent terminology)
 
-(defun howm-item-range (item)
-  "List of beginning-place and end-place of paragraph to which ITEM belongs."
+(defun howm-item-with-temp-buffer (item proc)
   (with-temp-buffer
     (howm-page-insert (howm-item-page item))
     (let* ((p (howm-item-place item))
@@ -863,12 +903,52 @@ This feature does not work when `howm-view-search-in-result-correctly' is nil."
                 (progn
                   (riffle-set-place p)
                   (howm-view-paragraph-region)))))
-      (list (progn
-              (goto-char (car r))
-              (riffle-get-place))
-            (progn
-              (goto-char (second r))
-              (riffle-get-place))))))
+      (narrow-to-region (car r) (second r))
+      (funcall proc item))))
+
+(defun howm-item-titles (item)
+"List of titles of ITEM.
+When place (see `howm-item-place') is specified, ITEM has at most one title.
+Otherwise, ITEM can have two or more titles."
+  (howm-item-with-temp-buffer
+   item
+   (lambda (i)
+     (let ((titles nil))
+       (goto-char (point-min))
+       (while (re-search-forward (howm-list-title-regexp) nil t)
+         (setq titles
+               (cons (buffer-substring-no-properties (match-beginning 0)
+                                                     (line-end-position))
+                     titles)))
+       (reverse titles)))))
+
+(defun howm-item-range (item)
+  "List of beginning-place and end-place of paragraph to which ITEM belongs."
+  (howm-item-with-temp-buffer
+   item
+   (lambda (i)
+     (let ((r (list (point-min) (point-max))))
+       (widen)
+       (list (progn
+               (goto-char (car r))
+               (riffle-get-place))
+             (progn
+               (goto-char (second r))
+               (riffle-get-place)))))))
+;;   (with-temp-buffer
+;;     (howm-page-insert (howm-item-page item))
+;;     (let* ((p (howm-item-place item))
+;;            (r (if (null p)
+;;                   (list (point-min) (point-max))
+;;                 (progn
+;;                   (riffle-set-place p)
+;;                   (howm-view-paragraph-region)))))
+;;       (list (progn
+;;               (goto-char (car r))
+;;               (riffle-get-place))
+;;             (progn
+;;               (goto-char (second r))
+;;               (riffle-get-place))))))
 
 (defun howm-item-list-rangeset (item-list)
   "Make assoc list of page to rangeset.
@@ -906,8 +986,8 @@ list of items in ITEM-LIST which do not satisfy the above condition."
                           (if (howm-item-place item)
                               (list item)
                             (let ((f (howm-make-folder-from-items (list item))))
-                              (or (howm-view-search-folder-items (howm-view-title-regexp-grep)
-                                                                 f)
+                              (or (howm-view-search-folder-items
+                                   (howm-view-title-regexp-grep) f)
                                   (list item)))))
                         item-list))
   (let* ((alist (howm-item-list-rangeset reference-item-list))
