@@ -1,7 +1,7 @@
 ;;; riffle.el --- template of list browser with immediate preview
-;;; Copyright (c) 2004, 2005
+;;; Copyright (c) 2004, 2005, 2006
 ;;;   by HIRAOKA Kazuyuki <khi@users.sourceforge.jp>
-;;; $Id: riffle.el,v 1.23 2005/08/02 15:34:30 hira Exp $
+;;; $Id: riffle.el,v 1.25 2006/01/06 15:35:46 hira Exp $
 ;;;
 ;;; This program is free software; you can redistribute it and/or modify
 ;;; it under the terms of the GNU General Public License as published by
@@ -168,29 +168,6 @@ This function returns effective value of ITEM-LIST."
   (riffle-summary-show item-list)
   (riffle-summary-check t))
 
-;; (defun riffle-summary (&optional name item-list type)
-;;   (when (null name)
-;;     (setq name      (riffle-name)
-;;           item-list (riffle-item-list)
-;;           type      riffle-type))
-;;   (if (null item-list)
-;;       (progn
-;;         (or (riffle-summary-nop-p)
-;;             (message "No match"))
-;;         nil)
-;;     (let ((d default-directory))
-;;       (riffle-setup-buffer #'riffle-summary-name-format name item-list type)
-;;       (setq default-directory d)
-;;       (if (riffle-summary-nop-p)
-;;           t  ;; do nothing
-;;         (riffle-summary-subr name item-list)))))
-
-;; (defun riffle-summary-subr (name item-list)
-;;   (riffle-summary-set-mode)
-;;   (riffle-summary-show item-list)
-;;   (riffle-summary-check t)
-;;   t)
-
 (defun riffle-summary-show (item-list)
   (buffer-disable-undo)
   (setq buffer-read-only nil)
@@ -335,6 +312,9 @@ This function returns effective value of ITEM-LIST."
             n (+ n 1)))
     (min n (1- (length riffle-contents-end))))) ;; for the last line
 
+(defun riffle-contents-current-item ()
+  (nth (riffle-contents-item-number (point)) (riffle-item-list)))
+
 (defun riffle-contents-beginning (n)
   (nth n (cons 1 riffle-contents-end)))
 
